<?php
/* @(#) $Id: os_lib_alerts.php,v 1.6 2007/03/22 01:11:35 dcid Exp $ */

/* Copyright (C) 2006-2007 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


/* Creating results file */
function __os_createresults($out_file, $evt_count, $alert_list)
{
    /* Removing one from evt count */
    $evt_count--;

    
    /* Opening output file */
    $fp = fopen($out_file, "w");
    if(!$fp)
    {
        return(NULL);
    }
    
    
    /* Printing all available levels  */
    fwrite($fp, '<table width="100%">');
    search_pavailable($fp, $alert_list[0]{'levels'},
                      $alert_list[0]{'levels_total'},
                      "level", "severity", "Severity breakdown");

    /* Printing all available rules */
    search_pavailable($fp, $alert_list[0]{'ids'},
                      $alert_list[0]{'ids_total'},
                      "id", "rule", "Rules breakdown");


    /* Printing all available ips */
    search_pavailable($fp, $alert_list[0]{'srcips'},
                      $alert_list[0]{'srcips_total'},
                      "srcip", "Source IP", "Src IP breakdown");
    
    fwrite($fp,'<tr><td><br /><b>First event</b> at <a href="#ft">'.
                date('Y M d H:i:s', $alert_list[$evt_count]{'time'}).
               '</a></td></tr>
                <tr><td><b>Last event</b> at <a href="#lt">'.
                date('Y M d H:i:s', $alert_list[1]{'time'}).
               '</a></td></tr>
                </table><br /><h2>Alert list</h2>
                <a name="ft" />
               ');


    while($evt_count > 0)
    {
        $alert = $alert_list[$evt_count];
        $al_date = date('Y M d H:i:s', $alert{'time'});

        /* Printing level block */
        fwrite($fp, '<div id="ctlevel'.$alert{'level'}.'-'.
                    $alert_list[0]{'levels_total'}{$alert{'level'}}--.
                    '" style="display: block">');

        /* Printing id block */
        fwrite($fp, '<div id="ctid'.$alert{'id'}.'-'.
                    $alert_list[0]{'ids_total'}{$alert{'id'}}--.
                    '" style="display: block">');

        /* Printing srcip block */
        fwrite($fp, '<div id=\'ctsrcip'.$alert{'srcip'}.'-'.
                    $alert_list[0]{'srcips_total'}{$alert{'srcip'}}.
                    '\' style="display: block">');

        fwrite($fp, "<ul class=\"alert\"><b>".$al_date
            .'</b> Rule Id: <a
            href="http://www.ossec.net/wiki/index.php/Rule:'.$alert{'id'}.'">'
            .$alert{'id'}.'</a>'
            ." level: ".$alert{'level'}."\n<br />");
        fwrite($fp, "<b>Location: </b>".$alert{'location'});
        if($alert{'srcip'} != "(none)")
        {
            fwrite($fp,"<br /><b>Src IP</b>: ".$alert{'srcip'});
        }
        fwrite($fp,
            '<br /><b>'.$alert{'description'}.'</b>'.
            '</ul><ul class="msg">'."\n");
        foreach($alert{'msg'} as $alert_msg)
        {
            if($alert_msg == NULL)
                break;

            fwrite($fp, $alert_msg."<br />\n");
        }
        fwrite($fp, "<br /></ul></div></div></div>\n");
        $evt_count--;
    }
}


/* Internal function to parse alerts. */
function __os_parsealert(&$fp, $curr_time, 
                         $init_time, $final_time, $min_level,
                         $rule_id, $location_pattern, 
                         $str_pattern, $group_pattern, $group_regex,
                         $srcip_pattern, $user_pattern, 
                         $log_pattern, $log_regex, $rc_code_hash)
{
    $evt_time = 0;
    $evt_id = 0;
    $evt_level = 0;
    $evt_description = NULL;
    $evt_location = NULL;
    $evt_srcip = NULL;
    $evt_user = NULL;
    $evt_group = NULL;
    $evt_msg[0] = "";
    
    while(!feof($fp)) 
    {
        $buffer = fgets($fp, 1024);
        
        /* Getting event header */
        if(strncmp($buffer, "** Alert", 8) != 0)
        {
            continue;
        }

        
        /* Getting event time */
        $evt_time = substr($buffer, 9, 10);
        if(is_numeric($evt_time) === FALSE)
        {
            $evt_time = 0;
            continue;
        }

        /* Checking if event time is in the timeframe */
        if(($init_time != 0) && ($evt_time < $init_time))
        {
            continue;
        }

        if(($final_time != 0) && ($evt_time > $final_time))
        {
            return(NULL);
        }


        /* Getting group information */
        $evt_group = strstr($buffer, "-");
        if($evt_group === FALSE)
        {
            /* Invalid group */
            continue;
        }
        
        
        /* Filtering based on the group */
        if($group_pattern != NULL)
        {
            if(strpos($evt_group, $group_pattern) === FALSE)
            {
                continue;
            }
        }
        else if($group_regex != NULL)
        {
            if(!preg_match($group_regex, $evt_group))
            {
                continue;
            }
        }

        /* Getting log formats */
        if($log_pattern != NULL)
        {
            if(strpos($evt_group, $log_pattern) === FALSE)
            {
                continue;
            }
        }
        else if($log_regex != NULL)
        {
            if(!preg_match($log_regex, $evt_group))
            {
                continue;
            }
        }

        

        /* Getting location */
        $buffer = fgets($fp, 1024);
        $evt_location = substr($buffer, 21);
        if($location_pattern)
        {
            if(strpos($evt_location, $location_pattern) === FALSE)
            {
                if(!$rc_code_hash{'location_pattern'})
                    continue;
            }
            else
            {
                if($rc_code_hash{'location_pattern'})
                    continue;
            }
        }


        /* Getting rule, level and description */
        $buffer = fgets($fp, 1024);
        $token = strtok($buffer, " ");
        if($token === FALSE)
        {
            continue;
        }
        
        
        /* Rule id */
        $token = strtok(" ");
        $evt_id = $token;
        if(is_numeric($evt_id) === FALSE)
        {
            continue;
        }

        /* Checking rule id */
        if($rule_id != NULL)
        {
            if(!preg_match($rule_id, $evt_id))
            {
                continue;
            }
        }
        
        
        /* Level */
        $token = strtok(" ");
        $token = strtok(" ");
        $evt_level = $token;
        $evt_level = rtrim($evt_level, ")");
        if(is_numeric($evt_level) === FALSE)
        {
            continue;
        }

        /* Checking event level */
        if($evt_level < $min_level)
        {
            continue;
        }

        /* Getting description */
        $token = strtok("'");
        $token = strtok("'");
        $evt_description = $token;


        /* srcip */
        $buffer = fgets($fp, 1024);
        $buffer = rtrim($buffer);
        $evt_srcip = substr($buffer, 8);
        
        if($srcip_pattern != NULL)
        {
            if(strpos($evt_srcip,$srcip_pattern) === FALSE)
            {
                if(!$rc_code_hash{'srcip_pattern'})
                    continue;
            }
            else
            {
                if($rc_code_hash{'srcip_pattern'})
                    continue;
            }
        }
        

        /* user */
        $buffer = fgets($fp, 1024);
        $buffer = rtrim($buffer);
        if($buffer != "User: (none)")
        {
            $evt_user = substr($buffer, 6);
            if($evt_user == "SYSTEM")
            {
                $evt_user = NULL;
            }
        }
        if($user_pattern)
        {
            if(($evt_user == NULL) || 
               (strpos($evt_user, $user_pattern) === FALSE))
            {
                if(!$rc_code_hash{'user_pattern'})
                    continue;
            }
            else
            {
                if($rc_code_hash{'user_pattern'})
                    continue;
            }
        }
                                            
        

        /* message */
        $buffer = fgets($fp, 2048);
        $msg_id = 0;
        $evt_msg[$msg_id] = NULL;
        $pattern_matched = 0;
        while(strlen($buffer) > 3)
        {
            if($buffer == "\n")
            {
                break;
            }

            if(($str_pattern != NULL) && 
               (strpos($buffer, $str_pattern) !== FALSE))
            {
                $pattern_matched = 1;
            }

            $evt_msg[$msg_id] = rtrim($buffer);
            $evt_msg[$msg_id] = ereg_replace("<", "&lt;", $evt_msg[$msg_id]);
            $evt_msg[$msg_id] = ereg_replace(">", "&gt;", $evt_msg[$msg_id]);
            $buffer = fgets($fp, 2048);
            $msg_id++;
            $evt_msg[$msg_id] = NULL;
        }

        /* Searching by pattern */
        if($str_pattern != NULL && $pattern_matched == 0 && 
           $rc_code_hash{'str_pattern'})
        {
            $evt_srcip = NULL;
            $evt_user = NULL;
            continue;
        }
        else if(!$rc_code_hash{'str_pattern'} && $pattern_matched == 1)
        {
            $evt_srcip = NULL;
            $evt_user = NULL;
            continue;
        }


        /* If we reach here, we got a full alert */
        $alert_struct{'time'} = $evt_time;
        $alert_struct{'id'} = $evt_id;
        $alert_struct{'level'} = $evt_level;
        
        $evt_user = ereg_replace("<", "&lt;", $evt_user);
        $evt_user = ereg_replace(">", "&gt;", $evt_user);
        $alert_struct{'user'} = $evt_user;

        
        $evt_srcip = ereg_replace("<", "&lt;", $evt_srcip);
        $evt_srcip = ereg_replace(">", "&gt;", $evt_srcip);
        $alert_struct{'srcip'} = $evt_srcip;
        
        $alert_struct{'description'} = $evt_description;
        $alert_struct{'location'} = $evt_location;
        $alert_struct{'msg'} = $evt_msg;

        return($alert_struct);
    }

    return(NULL);
}


function os_searchalerts($ossec_handle, $search_id,
                         $init_time, $final_time, 
                         $max_count,
                         $min_level,
                         $rule_id,
                         $location_pattern,
                         $str_pattern,
                         $group_pattern,
                         $srcip_pattern,
                         $user_pattern,
                         $log_pattern)
{
    $alert_list = NULL;
    $alert_count = 1;
    
    $file_count = 0;
    $file_list[0] = NULL;

    $output_count = 1;
    $output_file[0] = NULL;
    $output_file[1] = NULL;

    $curr_time = time(0);


    /* Clearing arguments */
    if($rule_id != NULL)
    {
        $rule_id = "/".$rule_id."/";
    }

    if(strpos($group_pattern,"|") !== FALSE)
    {
        $group_regex = "/".$group_pattern."/";
        $group_pattern = NULL;
    }

    if(strpos($log_pattern,"|") !== FALSE)
    {
        $log_regex = "/".$log_pattern."/";
        $log_pattern = NULL;
    }
    
    /* Setting rc code */
    if(($user_pattern != NULL) && ($user_pattern[0] == '!'))
    {
        $user_pattern = substr($user_pattern, 1);
        $rc_code_hash{'user_pattern'} = TRUE;
    }
    else
    {
        $rc_code_hash{'user_pattern'} = FALSE;
    }

    /* str */
    if(($str_pattern != NULL) && ($str_pattern[0] == '!'))
    {
        $str_pattern = substr($str_pattern, 1);
        $rc_code_hash{'str_pattern'} = FALSE;
    }
    else
    {
        $rc_code_hash{'str_pattern'} = TRUE;
    }

    /* srcip */
    if(($srcip_pattern != NULL) && ($srcip_pattern[0] == '!'))
    {
        $srcip_pattern = substr($srcip_pattern, 1);
        $rc_code_hash{'srcip_pattern'} = TRUE;
    }
    else
    {
        $rc_code_hash{'srcip_pattern'} = FALSE;
    }
    
    /* location */
    if(($location_pattern != NULL) && ($location_pattern[0] == '!'))
    {
        $location_pattern = substr($location_pattern, 1);
        $rc_code_hash{'location_pattern'} = TRUE;
    }
    else
    {
        $rc_code_hash{'location_pattern'} = FALSE;
    }
    

    /* Cleaning old entries */
    os_cleanstored(NULL);


    /* Getting first file */
    $init_loop = $init_time;
    while($init_loop <= $final_time)
    {
        $l_year_month = date('Y/M',$init_loop);
        $l_day = date('d',$init_loop);
        
        $file_list[$file_count] = "logs/alerts/".
                                  $l_year_month."/ossec-alerts-".$l_day.".log";

        /* Adding one day */
        $init_loop+=86400;
        $file_count++;
    }
    
    
    /* Getting each file */
    foreach($file_list as $file)
    {
        $log_file = $ossec_handle{'dir'}.'/'.$file;
        $fp = false;
        $new_file = NULL;


        /* Opening alert file */
        $fp = fopen($log_file, 'r');
        if($fp === false)
        {
            /* If file does not exist, try compressed form */
            $log_file = $ossec_handle{'dir'}.'/'.$file.".gz";
            $fp_gz = gzopen($log_file, 'r');
            if($fp_gz === false)
            {
                continue;
            }

            /* Writting file in clear text */
            $new_file = "./tmp/file-tmp.".posix_getpid().$curr_time.".php";
            $fp = fopen($new_file, "w");
            if($fp === false)
            {
                gzclose($fp_gz);
                continue;
            }
            
            while (!gzeof($fp_gz)) 
            {
                fputs($fp, gzread($fp_gz, 10000));
            }
            gzclose($fp_gz);

            /* Closing file and opening it for reading */
            fclose($fp);
            $fp = fopen($new_file, "r");
            if($fp === false)
            {
                /* Unable to open new file. */
                unlink($new_file);
                continue;
            }
        }


        /* Reading all the entries */
        while(1)
        {
            /* Dont get more than max count alerts per page */
            if($alert_count >= $max_count)
            {
                $output_file[$output_count] = "./tmp/output-tmp.".
                                            $output_count."-".$alert_count."-".
                                            $search_id.".php";
                
                __os_createresults($output_file[$output_count],
                                   $alert_count, $alert_list); 

                $output_file[0]{$output_count} = $alert_count -1;
                $alert_count = 1;
                $alert_list = NULL;
                $output_count++;
                $output_file[$output_count] = NULL;
            }
            
            $alert_hash = __os_parsealert($fp, $curr_time, $init_time, 
                                          $final_time, $min_level,
                                          $rule_id, $location_pattern,
                                          $str_pattern, $group_pattern,
                                          $group_regex,
                                          $srcip_pattern, $user_pattern,
                                          $log_pattern, $log_regex,
                                          $rc_code_hash);
            if($alert_hash == NULL)
            {
                break;
            }


            /* Couting by level, rule and srcip */
            $alert_list[0]{'levels_total'}{$alert_hash{'level'}}++;
            $alert_list[0]{'srcips_total'}{$alert_hash{'srcip'}}++;
            $alert_list[0]{'ids_total'}{$alert_hash{'id'}}++;

            $alert_list[0]{'ids'}{$alert_hash{'id'}}++;
            $alert_list[0]{'srcips'}{$alert_hash{'srcip'}}++;
            $alert_list[0]{'levels'}{$alert_hash{'level'}}++;

            $output_file[0]{'count'}++;


            /* Adding alert */
            $alert_list[$alert_count] = $alert_hash;
            $alert_count++;
        }

        /* Closing file */
        if($new_file != NULL)
        {
            unlink($new_file);
        }
        fclose($fp);
    }


    /* Creating last entry */
    $output_file[$output_count] = "./tmp/output-tmp.".
                                  $output_count."-".$alert_count."-".
                                  $search_id.".php";
    
    $output_file[0]{$output_count} = $alert_count -1;
    $output_file[$output_count +1] = NULL;

     __os_createresults($output_file[$output_count],
                        $alert_count, $alert_list);                                  

    $output_file[0]{'pg'} = $output_count;
    return($output_file);
}


function os_cleanstored($search_id)
{
    if($search_id != NULL)
    {
        foreach (glob("./tmp/output-tmp.*-*-".$search_id.".php") as $filename)
        {
            unlink($filename);
        }
    }
    else
    {
        foreach (glob("./tmp/*.php") as $filename)
        {
            if(filemtime($filename) < (time(0) - 1800))
            {
                unlink($filename);
            }
        }
    }
}


function os_getstoredalerts($ossec_handle, $search_id)
{
    $output_file[0] = NULL;
    $output_file[1] = NULL;
    $output_count = 1;


    /* Cleaning old entries */
    os_cleanstored(NULL);
    
    
    $filepattern = "/^\.\/tmp\/output-tmp\.(\d{1,3})-(\d{1,6})-\d+\.php$/";
    
    foreach (glob("./tmp/output-tmp.*-*-".$search_id.".php") as $filename) 
    {
        if(preg_match($filepattern, $filename, $regs))
        {
            $page_n = $regs[1];
            $alert_count = $regs[2] -1;
        }
        else
        {
            continue;
        }

        if($page_n >= 1 && $page_n < 512)
        {
            $output_file[$page_n] = $filename;
            $output_file[0]{$page_n} = $alert_count;
        
            $output_file[$page_n +1] = NULL;
            $output_file[0]{'count'} += $alert_count;
            
            $output_count++;
        }
    }

    $output_file[0]{'pg'} = $output_count -1;

    return($output_file);
}



function os_getalerts($ossec_handle, $init_time, $final_time, $max_count)
{
    $file = NULL;
    $alert_list = NULL;
    $alert_count = 0;

    $curr_time = time(0);
    
    
    /* Checking if agent_dir is set */
    if(!isset($ossec_handle{'dir'})||($ossec_handle{'dir'}==NULL))
    {
        $ossec_handle{'error'} = "Unable to open ossec dir: ".
                                  $ossec_handle{'dir'};
        return(NULL);
    }


    /* Getting log dir */
    $log_file = $ossec_handle{'dir'}.'/logs/alerts/alerts.log';
    

    /* Opening alert file */
    $fp = fopen($log_file, 'r');
    if($fp === false)
    {
        $ossec_handle{'error'} = "Unable to open log file: ".$log_file;
        return(NULL);
    }


    /* If times are set to zero, we monitor the last *count files. */
    if(($init_time == 0) && ($final_time == 0))
    {
        clearstatcache();
        os_cleanstored();

        
        /* Getting file size */            
        $f_size = filesize($log_file);
        
        /* Average size of every event: 300-350 */
        $f_point = $max_count * 325;
        
        
        /* If file size is large than the counter fseek to the
         * average place in the file.
         */
        if($f_size > $f_point)
        {
            $seek_place = $f_size - $f_point;
            fseek($fp, $seek_place, "SEEK_SET");
        }
    }
    

    /* Getting alerts */
    $count = 0;
    while(1) 
    {
        $alert_hash = __os_parsealert($fp, $curr_time, 
                                      $init_time, $final_time, 0, NULL, NULL,
                                      NULL, NULL,
                                      NULL, NULL, NULL, NULL, NULL, NULL, NULL);
        if($alert_hash == NULL)
        {
            break;
        }

        $alert_list[$count] = $alert_hash;
        $count++;
    }    

    fclose($fp);
    return($alert_list);
}


?>
