<?php
/* @(#) $Id: os_lib_firewall.php,v 1.1 2007/03/01 03:52:27 dcid Exp $ */

/* Copyright (C) 2006-2007 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */
       


/* Internal function to parse firewall messages. */
function __os_parsefw(&$fp, $file_month, $curr_time, 
                       $init_time, $final_time, $protocol_pattern,
                       $srcip_pattern, $dstip_pattern,
                       $srcport_pattern, $dstport_pattern,
                       $action_pattern, $location_pattern)
{
    $evt_time = 0;
    $evt_protocol = NULL;
    $evt_location = NULL;
    $evt_action = NULL;
    $evt_srcip = NULL;
    $evt_dstip = NULL;
    $evt_srcport = NULL;
    $evt_dstport = NULL;
    $evt_msg = NULL;
    
    /* Firewall pattern */
    $fwpattern = "/^([0-9]{4}) [a-zA-Z]{3} ([0-9]{2}) ".
                 "([0-9]{2}):([0-9]{2}):([0-9]{2}) ".
                 "([^>]+>\S+) (\w+) (\w+) ([a-zA-Z0-9\._-]+):".
                 "([a-zA-Z0-9_-]+)->([a-zA-Z0-9\._-]+):([a-zA-Z0-9_-]+)$/";
                   
    while(!feof($fp)) 
    {
        $buffer = fgets($fp, 2048);
        $buffer = rtrim($buffer);
       
        if(preg_match($fwpattern, $buffer, $regs))
        {
            $evt_time = mktime($regs[3], $regs[4], $regs[5], 
                               $file_month, $regs[2], $regs[1]);
            $evt_location = $regs[6];
            $evt_action = $regs[7];
            $evt_protocol = $regs[8];
            $evt_srcip = $regs[9];
            $evt_srcport = $regs[10];
            $evt_dstip = $regs[11];
            $evt_dstport = $regs[12];
        }
        else
        {
            continue;
        }
        

        /* Checking if event time is in the timeframe */
        if(($init_time != 0) && ($evt_time < $init_time))
        {
            continue;
        }

        if(($final_time != 0) && ($evt_time > $final_time))
        {
            return(NULL);
        }


        
        /* If we reach here, we got the event */
        $alert_struct{'time'} = $evt_time;
        $alert_struct{'location'} = $evt_location;
        $alert_struct{'action'} = $evt_action;
        $alert_struct{'msg'} = $buffer;
        $alert_struct{'srcip'} = $evt_srcip;
        $alert_struct{'dstip'} = $evt_dstip;
        $alert_struct{'srcport'} = $evt_srcport;
        $alert_struct{'dstport'} = $evt_dstport;
        $alert_struct{'protocol'} = $evt_protocol;

        return($alert_struct);
    }

    return(NULL);
}


function os_searchfw($ossec_handle, $search_id,
                     $init_time, $final_time,
                     $max_count, $protocol_pattern,
                     $srcip_pattern, $dstip_pattern,
                     $srcport_pattern, $dstport_pattern,
                     $action_pattern, $location_pattern) 
{
    $alert_list = NULL;
    $alert_count = 1;
    $file_list[0] = NULL;
    $file_count = 0;

    $curr_time = time(0);


    /* Saved values for java script optmization */
    $s_srcip = "-1";
    $s_dstip = "-1";
    $s_srcport = "-1";
    $s_dstport = "-1";


    /* Getting first file */
    $init_loop = $init_time;
    while($init_loop <= $final_time)
    {
        $l_year_month = date('Y/M',$init_loop);
        $l_day = date('d',$init_loop);
        $l_month = date('n', $init_loop);
        
        $file_list[$file_count] = "logs/firewall/".
                                  $l_year_month."/ossec-firewall-".
                                  $l_day.".log";

        /* Adding one day */
        $init_loop+=86400;
        $file_count++;
    }
    
    
    /* Getting each file */
    foreach($file_list as $file)
    {
        $log_file = $ossec_handle{'dir'}.'/'.$file;
        $fp = false;
        $new_file = NULL;


        /* Opening alert file */
        $fp = fopen($log_file, 'r');
        if($fp === false)
        {
            /* If file does not exist, try compressed form */
            $log_file = $ossec_handle{'dir'}.'/'.$file.".gz";
            $fp_gz = gzopen($log_file, 'r');
            if($fp_gz === false)
            {
                continue;
            }

            /* Writting file in clear text */
            $new_file = "./tmp/filefw-tmp.".posix_getpid().$curr_time.".php";
            $fp = fopen($new_file, "w");
            if($fp === false)
            {
                gzclose($fp_gz);
                continue;
            }
            
            while (!gzeof($fp_gz)) 
            {
                fputs($fp, gzread($fp_gz, 10000));
            }
            gzclose($fp_gz);

            /* Closing file and opening it for reading */
            fclose($fp);
            $fp = fopen($new_file, "r");
            if($fp === false)
            {
                /* Unable to open new file. */
                unlink($new_file);
                continue;
            }
        }

        while(1)
        {
            /* Dont get more than max count alerts */
            if($alert_count > $max_count)
            {
                break;
            }
            
            $alert_hash = __os_parsefw($fp, $l_month, $curr_time, $init_time, 
                                       $final_time, $protocol_pattern,
                                       $srcip_pattern, $dstip_pattern,
                                       $srcport_pattern, $dstport_pattern,
                                       $action_pattern, $location_pattern);
            if($alert_hash == NULL)
            {
                break;
            }

            /* Adding information about alert */

            /* Optmizing based on the dstip */
            if($s_dstip != $alert_hash{'dstip'})
            {
                if(!isset($alert_list[0]{'dstips'}{$alert_hash{'dstip'}}))
                {
                    $alert_list[0]{'dstips'}{$alert_hash{'dstip'}} = 1;
                }
                else
                {
                    $alert_list[0]{'dstips'}{$alert_hash{'dstip'}}++;
                }
                $s_dstip = $alert_hash{'dstip'};
                $s_srcip = "-1";

                if($alert_count > 1)
                {
                    $alert_list[$alert_count -1]{'dstip_count'} =
                        $alert_list[0]{'dstips'}
                        {$alert_list[$alert_count -1]{'dstip'}};
                }
            }
            $alert_list[0]{'dstips_total'}{$alert_hash{'dstip'}}++;

            /* Optmizing based on srcip */
            if($s_srcip != $alert_hash{'srcip'})
            {
                if($alert_count > 1)
                {
                    $alert_list[$alert_count -1]{'srcip_count'} =
                        $alert_list[0]{'srcips'}
                        {$alert_list[$alert_count -1]{'srcip'}};
                }
                
                if(!isset($alert_list[0]{'srcips'}{$alert_hash{'srcip'}}))
                {
                    $alert_list[0]{'srcips'}{$alert_hash{'srcip'}} = 1;
                }
                else
                {
                    $alert_list[0]{'srcips'}{$alert_hash{'srcip'}}++;
                }
                $s_srcip = $alert_hash{'srcip'};
            }
            $alert_list[0]{'srcips_total'}{$alert_hash{'srcip'}}++;    
            
            $alert_list[$alert_count] = $alert_hash;
            $alert_count++;
        }


        /* Closing file */
        if($new_file != NULL)
        {
            unlink($new_file);
        }
        fclose($fp);
    }

    /* Checking if last entry needs to be marked */
    if($alert_list != NULL)
    {
        $alert_list[$alert_count -1]{'dstip_count'} =
            $alert_list[0]{'dstips'}
            {$alert_list[$alert_count -1]{'dstip'}};

        $alert_list[$alert_count -1]{'srcip_count'} =    
            $alert_list[0]{'srcips'}
            {$alert_list[$alert_count -1]{'srcip'}};
    }
    return($alert_list);
}

?>
